#coding: utf-8

import ConfigParser
import json
import importlib
import time
import os
import requests
import sys

from log import logger, setup_logging
from optparse import OptionParser


def collect_all():
    '''
    Собираем данные со всех информеров в сстеме.
    '''
    result = {}
    modules_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'informers')
    
    modules = [fn for fn in os.listdir(modules_path) if fn.endswith('.py')]

    for fn in modules:
        if fn == '__init__.py':
            continue

        name, ext = os.path.splitext(fn)
        module = importlib.import_module('.informers.{0}'.format(name), package=__package__)
        logger.debug('Trying to collect data from "{0}"...'.format(name))
        result[name] = module.collect()
    return result


def main():
    setup_logging()

    cfg_files = ['./garbo.cfg']
    parser = OptionParser()
    parser.add_option('-c', '--cfg', dest='cfg_file',
                      help='Load additional config file.', metavar='<CONFIG>')
    parser.add_option('-d', '--dump', dest='just_dump', action='store_true',
                      help='Just dump result to stdout.')
    (options, args) = parser.parse_args()

    if options.cfg_file:
        cfg_files.append(options.cfg_file)

    config = ConfigParser.SafeConfigParser()
    if not config.read(cfg_files):
        logger.error('None of configuration files were parsed: {0}'.format(', '.join(cfg_files)))
        return        

    client = config.get('identity', 'client')
    region = config.get('identity', 'region')
    product = config.get('identity', 'product')
    try:
        instance = config.get('identity', 'instance')
    except ConfigParser.NoOptionError:
        instance = None

    # Даже если эти значения есть в конфиге, они все должны содержать в себе
    # какие-либо данные.
    if not all([client, region, product]):
        logger.error('You must fill out [identity] section in config file!')
        return

    packet = {
        'identity': {
            'region': region,
            'client': client,
            'product': product,
        },
        'timestamp': int(time.time())
    }
    if instance:
        packet['identity']['instance'] = instance        

    logger.debug('Packet header: {0}'.format(json.dumps(packet)))
    packet['info'] = collect_all()
    payload = json.dumps(packet, indent=4)

    if options.just_dump:
        print payload
    else:
        # Отправляем собранные данные на удаленный сервер
        try:
            url = config.get('endpoint', 'url')
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) as e:
            url = 'http://localhost:1942/'

        if not url:
            logger.error("Endpoint URL not specified!")
            return

        url += 'register'
        headers = {'content-type': 'application/json'}
        logger.debug('Trying to send data ({0})...'.format(url))

        try:
            resp = requests.post(url, data=payload, headers=headers)
            reply = resp.json()
            if reply['result'] == 'OK':
                logger.info('Succesfully sent info.')
            else:
                logger.error('Something went wrong: {0}'.format(reply['result']))
        except requests.exceptions.RequestException as re:
            logger.error("Can't send data: {0}".format(unicode(re)))
        except (ValueError, IndexError) as e:
            logger.error('Server replied with an error: {0}'.format(text))            

if __name__ == '__main__':
    main()
